using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using Microsoft.Ink;

namespace JournalBrowser {
    /// <summary>
    /// A basic journal document class
    /// </summary>
    public class JournalDocument {

        public Ink GetInk(int i) {
            return Pages[0].Ink;
        }

        // For now, the Journal Document is just a list of pages.  The stationery is ignored.
        List<JournalPage> pages;
        public List<JournalPage> Pages {
            get { return pages; }
        }

        
        public JournalDocument(Stream xmlStream){ 
            pages = new List<JournalPage>();
            System.Xml.XmlDocument xmlDoc = new System.Xml.XmlDocument();
            xmlDoc.Load(xmlStream);
 
            string nodeName = xmlDoc.DocumentElement.Name;
            if (!nodeName.ToLower().Equals("journaldocument"))
                throw new JournalDocumentException();

            foreach (XmlNode child in xmlDoc.DocumentElement.ChildNodes) {
                string childName = child.Name;
                if (childName.ToLower().Equals("journalpage")) {
                    pages.Add(new JournalPage(child));
                }
                else if (childName.ToLower().Equals("stationery")) {
                }


            }
        }

        public int PageCount {
            get { return pages.Count; }
        }

    }

    public class JournalDocumentException : Exception {
    }

    public class JournalPage {

        Ink ink;
        public Ink Ink {
            get { return ink;  }
        }

        private int pageNumber;
        public int PageNumber {
            get { return pageNumber; }
        }

        List<JournalDrawing> drawings;
        List<JournalParagraph> paragraphs;

        public JournalPage(XmlNode xmlNode) {
            this.ink = new Ink();
            this.drawings = new List<JournalDrawing>();
            this.paragraphs = new List<JournalParagraph>();

            string nodeName = xmlNode.Name;
            if (! nodeName.ToLower().Equals("journalpage"))
                throw new JournalDocumentException();
            foreach (XmlNode node in xmlNode.ChildNodes) {
                string name = node.Name;
                if (name.ToLower().Equals("content")) {
                    AddPageContent(node);
                }
                else if (name.ToLower().Equals("docimage")) {
                }
                else if (name.ToLower().Equals("titleinfo")) {
                }
                else if (name.ToLower().Equals("stationery")) {
                }


            }
             

        }

        private void AddPageContent(XmlNode contentNode) {
            string nodeName = contentNode.Name;
            if (!nodeName.ToLower().Equals("content"))
                throw new JournalDocumentException();
            foreach (XmlNode node in contentNode.ChildNodes) {
                string name = node.Name;
                if (name.ToLower().Equals("paragraph")) {
                    JournalParagraph jp = new JournalParagraph(node);
                    this.paragraphs.Add(jp);
                    InkUtility.MergeInk(this.ink, jp.Ink);
                }
                else if (name.ToLower().Equals("drawing")) {
                    JournalDrawing jd = new JournalDrawing(node);
                    this.drawings.Add(jd);
                    InkUtility.MergeInk(this.ink, jd.Ink);
                }
                else if (name.ToLower().Equals("groupnode")) {
                }
                else if (name.ToLower().Equals("text")) {
                }
                else if (name.ToLower().Equals("image")) {
                }
                else if (name.ToLower().Equals("flag")) {
                }


            }
        }

        public override string ToString() {
            StringBuilder sb = new StringBuilder();

            sb.Append("Text : " + "\r\n");
            foreach (JournalParagraph para in this.paragraphs) {
                sb.Append(para.ToString());
                sb.Append("\r\n");
            }

            sb.Append("\r\n\r\nDrawings : " + "\r\n");
            foreach (JournalDrawing drawing in this.drawings) {
                sb.Append(drawing.ToString());
                sb.Append("\r\n");
            }

            return sb.ToString();

        }
    }

    public class JournalParagraph {

        Ink ink;
        public Ink Ink {
            get { return ink; }
        }

        List<JournalLine> lines;
        public List<JournalLine> Lines {
            get { return lines; }
        }

        public JournalParagraph(XmlNode paragraphNode) {
            this.ink = new Ink();
            lines = new List<JournalLine>();
            foreach (XmlNode node in paragraphNode.ChildNodes) {
                string name = node.Name;
                if (name.ToLower().Equals("line")) {
                    JournalLine jl = new JournalLine(node);
                    this.lines.Add(jl);
                    InkUtility.MergeInk(this.ink, jl.Ink);
                }
                else if (name.ToLower().Equals("paragraphlinemetrics")) {
                    // Just ignore this for now
                }
            }
        }

        public override string ToString() {
            StringBuilder sb = new StringBuilder();
            foreach (JournalLine jl in lines)
                sb.Append(jl.ToString() + "\r\n");
            return sb.ToString();
        }

    }

    public class JournalLine {
       
        Ink ink;
        public Ink Ink {
            get { return ink; }
        }

        List<JournalInkWord> words;
        public List<JournalInkWord> Words {
            get { return words; }
        }

        public JournalLine(XmlNode lineNode) {
            this.ink = new Ink();
            this.words = new List<JournalInkWord>();
            foreach (XmlNode node in lineNode.ChildNodes) {
                string name = node.Name;
                if (name.ToLower().Equals("inkword")) {
                    JournalInkWord jiw = new JournalInkWord(node);
                    this.words.Add(jiw);
                    InkUtility.MergeInk(this.ink, jiw.Ink);
                }
            }
        }

        public override string ToString() {
            StringBuilder sb = new StringBuilder();
            foreach (JournalInkWord jw in words)
                sb.Append(jw.ToString() + " ");
            return sb.ToString();
        }
    }

    public class JournalInkWord {
        string base64Ink;
        public string Base64Ink {
            get { return base64Ink; }
        }

        List<string> choices;
        public List<string> Choices {
            get { return choices; }
        }

        public string TopChoice {
            get { return (choices.Count == 0) ? "" : choices[0]; }
        }


        Ink ink;
        public Ink Ink {
            get { return ink; }
        }

        public JournalInkWord(XmlNode wordNode) {
            choices = new List<string>();
            this.ink = new Ink();
            foreach (XmlNode node in wordNode.ChildNodes) {
                string name = node.Name;
                if (name.ToLower().Equals("alternatelist")) {
                    choices = GetAlternateList(node);
                }
                else if (name.ToLower().Equals("inkobject")) {
                    base64Ink = node.InnerText;
                    ink = new Ink();
                    ink.Load(Convert.FromBase64String(base64Ink));
                }
                else if (name.ToLower().Equals("confidence")) {
                }
                else if (name.ToLower().Equals("scalartransform")) {
                }
                else if (name.ToLower().Equals("canreclassify")) {
                }
            }
        }

        List<string> GetAlternateList(XmlNode listNode) {
            List<string> list = new List<string>();
            foreach (XmlNode node in listNode.ChildNodes) {
                string name = node.Name;
                if (name.ToLower().Equals("alternate")) {
                    list.Add(node.InnerText);
                }
            }
            return list;
        }

        public override string ToString() {
            return this.TopChoice;
        }
    }

    public class JournalDrawing {
        int left;
        public int Left {
            get { return left; }
        }

        int top;
        public int Top {
            get { return top; }
        }

        int width;
        public int Width {
            get { return width; }
        }

        int height;
        public int Height {
            get { return height; }
        }

        string base64Ink;
        public string Base64Ink {
            get { return base64Ink; }
        }

        Ink ink;
        public Ink Ink {
            get { return ink; }
        }

        public JournalDrawing(XmlNode node) {
            string nodeName = node.Name;
            if (!nodeName.ToLower().Equals("drawing"))
                throw new JournalDocumentException();
            foreach (XmlAttribute attr in node.Attributes) {
                string attrName = attr.Name.ToLower();
                if (attrName.Equals("left"))
                    this.left = int.Parse(attr.Value);
                else if (attrName.Equals("top"))
                    this.top = int.Parse(attr.Value);
                else if (attrName.Equals("height"))
                    this.height = int.Parse(attr.Value);
                else if (attrName.Equals("width"))
                    this.width = int.Parse(attr.Value);  
            }

            // Look for an ink object - we will ignore other types of nodes
            foreach (XmlNode childNode in node.ChildNodes) {
                if (childNode.Name.ToLower().Equals("inkobject")) {
                    base64Ink = childNode.InnerText;
                    ink = new Ink();
                    ink.Load(Convert.FromBase64String(base64Ink));
                }

            }





        }

        public override string  ToString() {
            return "Drawing - left:" + left + "  top:" + top + "  height: " + height + "  width: " + width 
                + "\r\n" + this.Base64Ink;

        }
    }




}
